@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -36,6 +31,7 @@
 
 #include <utils/qtcassert.h>
 
+#include <QColor>
 #include <QDir>
 
 using namespace QmlJS;
@@ -527,6 +523,21 @@ public:
 
 };
 
+class UnsupportedTypesByQmlUi : public QStringList
+{
+public:
+    UnsupportedTypesByQmlUi()
+    {
+        (*this) << UnsupportedTypesByVisualDesigner()
+                << QLatin1String("Binding") << QLatin1String("ShaderEffect")
+                << QLatin1String("ShaderEffectSource") << QLatin1String("Canvas")
+                << QLatin1String("Component") << QLatin1String("Loader") << QLatin1String("Transition")
+                << QLatin1String("PropertyAnimation") << QLatin1String("SequentialAnimation")
+                << QLatin1String("ParallelAnimation") << QLatin1String("NumberAnimation");
+    }
+
+};
+
 class UnsupportedRootObjectTypesByVisualDesigner : public QStringList
 {
 public:
@@ -539,11 +550,24 @@ public:
 
 };
 
+class UnsupportedRootObjectTypesByQmlUi : public QStringList
+{
+public:
+    UnsupportedRootObjectTypesByQmlUi()
+    {
+        (*this) << UnsupportedRootObjectTypesByVisualDesigner()
+                << QLatin1String("Window") << QLatin1String("ApplicationWindow");
+    }
+
+};
+
 } // end of anonymous namespace
 
 Q_GLOBAL_STATIC(VisualAspectsPropertyBlackList, visualAspectsPropertyBlackList)
 Q_GLOBAL_STATIC(UnsupportedTypesByVisualDesigner, unsupportedTypesByVisualDesigner)
 Q_GLOBAL_STATIC(UnsupportedRootObjectTypesByVisualDesigner, unsupportedRootObjectTypesByVisualDesigner)
+Q_GLOBAL_STATIC(UnsupportedRootObjectTypesByQmlUi, unsupportedRootObjectTypesByQmlUi)
+Q_GLOBAL_STATIC(UnsupportedTypesByQmlUi, unsupportedTypesByQmlUi)
 
 Check::Check(Document::Ptr doc, const ContextPtr &context)
     : _doc(doc)
@@ -569,12 +593,13 @@ Check::Check(Document::Ptr doc, const ContextPtr &context)
     disableMessage(HintBinaryOperatorSpacing);
     disableMessage(HintOneStatementPerLine);
     disableMessage(HintExtraParentheses);
-    disableMessage(WarnImperativeCodeNotEditableInVisualDesigner);
-    disableMessage(WarnUnsupportedTypeInVisualDesigner);
-    disableMessage(WarnReferenceToParentItemNotSupportedByVisualDesigner);
-    disableMessage(WarnUndefinedValueForVisualDesigner);
-    disableMessage(WarnStatesOnlyInRootItemForVisualDesigner);
-    disableMessage(ErrUnsupportedRootTypeInVisualDesigner);
+
+    if (isQtQuick2Ui()) {
+        disableQmlDesignerChecks();
+    } else {
+        disableQmlDesignerChecks();
+        disableQmlDesignerUiFileChecks();
+    }
 }
 
 Check::~Check()
@@ -602,6 +627,48 @@ void Check::disableMessage(Type type)
     _enabledMessages.remove(type);
 }
 
+void Check::enableQmlDesignerChecks()
+{
+    enableMessage(WarnImperativeCodeNotEditableInVisualDesigner);
+    enableMessage(WarnUnsupportedTypeInVisualDesigner);
+    enableMessage(WarnReferenceToParentItemNotSupportedByVisualDesigner);
+    enableMessage(WarnAboutQtQuick1InsteadQtQuick2);
+    enableMessage(ErrUnsupportedRootTypeInVisualDesigner);
+    //## triggers too often ## check.enableMessage(StaticAnalysis::WarnUndefinedValueForVisualDesigner);
+}
+
+void Check::disableQmlDesignerChecks()
+{
+    disableMessage(WarnImperativeCodeNotEditableInVisualDesigner);
+    disableMessage(WarnUnsupportedTypeInVisualDesigner);
+    disableMessage(WarnReferenceToParentItemNotSupportedByVisualDesigner);
+    disableMessage(WarnUndefinedValueForVisualDesigner);
+    disableMessage(WarnStatesOnlyInRootItemForVisualDesigner);
+    disableMessage(ErrUnsupportedRootTypeInVisualDesigner);
+}
+
+void Check::enableQmlDesignerUiFileChecks()
+{
+    enableMessage(ErrUnsupportedRootTypeInQmlUi);
+    enableMessage(ErrUnsupportedTypeInQmlUi);
+    enableMessage(ErrFunctionsNotSupportedInQmlUi);
+    enableMessage(ErrBlocksNotSupportedInQmlUi);
+    enableMessage(ErrBehavioursNotSupportedInQmlUi);
+    enableMessage(ErrStatesOnlyInRootItemInQmlUi);
+    enableMessage(ErrReferenceToParentItemNotSupportedInQmlUi);
+}
+
+void Check::disableQmlDesignerUiFileChecks()
+{
+    disableMessage(ErrUnsupportedRootTypeInQmlUi);
+    disableMessage(ErrUnsupportedTypeInQmlUi);
+    disableMessage(ErrFunctionsNotSupportedInQmlUi);
+    disableMessage(ErrBlocksNotSupportedInQmlUi);
+    disableMessage(ErrBehavioursNotSupportedInQmlUi);
+    disableMessage(ErrStatesOnlyInRootItemInQmlUi);
+    disableMessage(ErrReferenceToParentItemNotSupportedInQmlUi);
+}
+
 bool Check::preVisit(Node *ast)
 {
     _chain.append(ast);
@@ -652,6 +719,10 @@ void Check::endVisit(UiObjectInitializer *)
 void Check::checkProperty(UiQualifiedId *qualifiedId)
 {
     const QString id = toString(qualifiedId);
+
+    if (id.isEmpty())
+        return;
+
     if (id.at(0).isLower()) {
         if (m_propertyStack.top().contains(id))
             addMessage(ErrPropertiesCanOnlyHaveOneBinding, fullLocationForQualifiedId(qualifiedId));
@@ -668,8 +739,11 @@ bool Check::visit(UiObjectDefinition *ast)
 bool Check::visit(UiObjectBinding *ast)
 {
     checkScopeObjectMember(ast->qualifiedId);
-    if (!ast->hasOnToken)
+    if (!ast->hasOnToken) {
         checkProperty(ast->qualifiedId);
+    } else {
+        addMessage(ErrBehavioursNotSupportedInQmlUi, locationFromRange(ast->firstSourceLocation(), ast->lastSourceLocation()));
+    }
 
     visitQmlObject(ast, ast->qualifiedTypeNameId, ast->initializer);
     return false;
@@ -712,6 +786,11 @@ static bool checkTypeForDesignerSupport(UiQualifiedId *typeId)
     return unsupportedTypesByVisualDesigner()->contains(getRightMostIdentifier(typeId)->name.toString());
 }
 
+static bool checkTypeForQmlUiSupport(UiQualifiedId *typeId)
+{
+    return unsupportedTypesByQmlUi()->contains(getRightMostIdentifier(typeId)->name.toString());
+}
+
 static bool checkTopLevelBindingForParentReference(ExpressionStatement *expStmt, const QString &source)
 {
     if (!expStmt)
@@ -739,19 +818,29 @@ void Check::visitQmlObject(Node *ast, UiQualifiedId *typeId,
 
     const SourceLocation typeErrorLocation = fullLocationForQualifiedId(typeId);
 
-    if (checkTypeForDesignerSupport(typeId))
-        addMessage(WarnUnsupportedTypeInVisualDesigner, typeErrorLocation);
-
-    if (m_typeStack.count() > 1 && getRightMostIdentifier(typeId)->name.toString() == QLatin1String("State"))
-        addMessage(WarnStatesOnlyInRootItemForVisualDesigner, typeErrorLocation);
-
     const QString typeName = getRightMostIdentifier(typeId)->name.toString();
 
+    if (checkTypeForDesignerSupport(typeId))
+        addMessage(WarnUnsupportedTypeInVisualDesigner, typeErrorLocation, typeName);
+
+    if (checkTypeForQmlUiSupport(typeId))
+        addMessage(ErrUnsupportedTypeInQmlUi, typeErrorLocation, typeName);
+
+    if (m_typeStack.count() > 1 && getRightMostIdentifier(typeId)->name.toString() == QLatin1String("State")) {
+        addMessage(WarnStatesOnlyInRootItemForVisualDesigner, typeErrorLocation);
+        addMessage(ErrStatesOnlyInRootItemInQmlUi, typeErrorLocation);
+    }
+
     if (m_typeStack.isEmpty()
             && unsupportedRootObjectTypesByVisualDesigner()->contains(typeName))
         addMessage(ErrUnsupportedRootTypeInVisualDesigner,
                    locationFromRange(ast->firstSourceLocation(), ast->lastSourceLocation()), typeName);
 
+    if (m_typeStack.isEmpty()
+            && unsupportedRootObjectTypesByQmlUi()->contains(typeName))
+        addMessage(ErrUnsupportedRootTypeInQmlUi,
+                   locationFromRange(ast->firstSourceLocation(), ast->lastSourceLocation()), typeName);
+
     bool typeError = false;
     if (_importsOk) {
         const ObjectValue *prototype = _context->lookupType(_doc.data(), typeId);
@@ -845,6 +934,8 @@ bool Check::visit(UiScriptBinding *ast)
             && checkTopLevelBindingForParentReference(cast<ExpressionStatement *>(ast->statement), _doc->source())) {
         addMessage(WarnReferenceToParentItemNotSupportedByVisualDesigner,
                    locationFromRange(ast->firstSourceLocation(), ast->lastSourceLocation()));
+        addMessage(ErrReferenceToParentItemNotSupportedInQmlUi,
+                   locationFromRange(ast->firstSourceLocation(), ast->lastSourceLocation()));
     }
 
     checkProperty(ast->qualifiedId);
@@ -1001,9 +1092,10 @@ bool Check::visit(FunctionDeclaration *ast)
 
 bool Check::visit(FunctionExpression *ast)
 {
+    SourceLocation locfunc = ast->functionToken;
+    SourceLocation loclparen = ast->lparenToken;
+
     if (ast->name.isEmpty()) {
-        SourceLocation locfunc = ast->functionToken;
-        SourceLocation loclparen = ast->lparenToken;
         if (locfunc.isValid() && loclparen.isValid()
                 && (locfunc.startLine != loclparen.startLine
                     || locfunc.end() + 1 != loclparen.begin())) {
@@ -1011,6 +1103,8 @@ bool Check::visit(FunctionExpression *ast)
         }
     }
 
+    addMessage(ErrFunctionsNotSupportedInQmlUi, locationFromRange(locfunc, loclparen));
+
     DeclarationsCheck bodyCheck;
     addMessages(bodyCheck(ast));
 
@@ -1110,6 +1204,8 @@ bool Check::visit(BinaryExpression *ast)
 
 bool Check::visit(Block *ast)
 {
+    addMessage(ErrBlocksNotSupportedInQmlUi, locationFromRange(ast->firstSourceLocation(), ast->lastSourceLocation()));
+
     if (Node *p = parent()) {
         if (!cast<UiScriptBinding *>(p)
                 && !cast<UiPublicMember *>(p)
@@ -1391,12 +1487,20 @@ void Check::warnAboutUnnecessarySuppressions()
 
 bool Check::isQtQuick2() const
 {
-    foreach (const Import &import, _imports->all()) {
-        if (import.info.name() == QLatin1String("QtQuick")
-                && import.info.version().majorVersion() == 2)
-            return true;
+    if (_doc->language() == Dialect::Qml) {
+        foreach (const Import &import, _imports->all()) {
+            if (import.info.name() == QLatin1String("QtQuick")
+                    && import.info.version().majorVersion() == 2)
+                return true;
+        }
+        return false;
     }
-    return false;
+    return _doc->language() == Dialect::QmlQtQuick2 || _doc->language() == Dialect::QmlQtQuick2Ui;
+}
+
+bool Check::isQtQuick2Ui() const
+{
+    return _doc->language() == Dialect::QmlQtQuick2Ui;
 }
 
 bool Check::visit(NewExpression *ast)
@@ -1443,12 +1547,20 @@ bool Check::visit(CallExpression *ast)
     // check for capitalized function name being called
     SourceLocation location;
     const QString name = functionName(ast->base, &location);
+
+    //We have to allow the qsTr function for translation.
+    if (name != QLatin1String("qsTr") && name != QLatin1String("qsTrId"))
+        addMessage(ErrFunctionsNotSupportedInQmlUi, location);
+
     if (!name.isEmpty() && name.at(0).isUpper()
             && name != QLatin1String("String")
             && name != QLatin1String("Boolean")
             && name != QLatin1String("Date")
             && name != QLatin1String("Number")
-            && name != QLatin1String("Object")) {
+            && name != QLatin1String("Object")
+            && name != QLatin1String("QT_TR_NOOP")
+            && name != QLatin1String("QT_TRANSLATE_NOOP")
+            && name != QLatin1String("QT_TRID_NOOP")) {
         addMessage(WarnExpectedNewWithUppercaseFunction, location);
     }
     if (cast<IdentifierExpression *>(ast->base) && name == QLatin1String("eval"))
