@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,37 +9,35 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
+
 #include "devicemanager.h"
 
 #include "idevicefactory.h"
 
 #include <coreplugin/icore.h>
+#include <coreplugin/messagemanager.h>
 #include <extensionsystem/pluginmanager.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorerconstants.h>
-#include <utils/qtcassert.h>
+#include <ssh/sshhostkeydatabase.h>
 #include <utils/fileutils.h>
 #include <utils/persistentsettings.h>
 #include <utils/qtcassert.h>
 #include <utils/portlist.h>
+#include <utils/algorithm.h>
 
 #include <QFileInfo>
 #include <QHash>
@@ -74,6 +72,7 @@ public:
     static DeviceManager *clonedInstance;
     QList<IDevice::Ptr> devices;
     QHash<Core::Id, Core::Id> defaultDevices;
+    QSsh::SshHostKeyDatabasePtr hostKeyDatabase;
 
     Utils::PersistentSettingsWriter *writer;
 };
@@ -135,6 +134,7 @@ void DeviceManager::save()
     QVariantMap data;
     data.insert(QLatin1String(DeviceManagerKey), toMap());
     d->writer->save(data, Core::ICore::mainWindow());
+    d->hostKeyDatabase->store(hostKeysFilePath());
 }
 
 void DeviceManager::load()
@@ -307,6 +307,11 @@ bool DeviceManager::isLoaded() const
     return d->writer;
 }
 
+QSsh::SshHostKeyDatabasePtr DeviceManager::hostKeyDatabase() const
+{
+    return d->hostKeyDatabase;
+}
+
 void DeviceManager::setDefaultDevice(Core::Id id)
 {
     QTC_ASSERT(this != instance(), return);
@@ -342,7 +347,14 @@ DeviceManager::DeviceManager(bool isInstance) : d(new DeviceManagerPrivate)
     if (isInstance) {
         QTC_ASSERT(!m_instance, return);
         m_instance = this;
-        connect(Core::ICore::instance(), SIGNAL(saveSettingsRequested()), SLOT(save()));
+        d->hostKeyDatabase = QSsh::SshHostKeyDatabasePtr::create();
+        const QString keyFilePath = hostKeysFilePath();
+        if (QFileInfo(keyFilePath).exists()) {
+            QString error;
+            if (!d->hostKeyDatabase->load(keyFilePath, &error))
+                Core::MessageManager::write(error);
+        }
+        connect(Core::ICore::instance(), &Core::ICore::saveSettingsRequested, this, &DeviceManager::save);
     }
 }
 
@@ -369,11 +381,9 @@ IDevice::Ptr DeviceManager::mutableDevice(Core::Id id) const
 
 bool DeviceManager::hasDevice(const QString &name) const
 {
-    foreach (const IDevice::Ptr &device, d->devices) {
-        if (device->displayName() == name)
-            return true;
-    }
-    return false;
+    return Utils::anyOf(d->devices, [&name](const IDevice::Ptr &device) {
+        return device->displayName() == name;
+    });
 }
 
 IDevice::ConstPtr DeviceManager::find(Core::Id id) const
@@ -416,6 +426,11 @@ IDevice::ConstPtr DeviceManager::fromRawPointer(const IDevice *device) const
     return fromRawPointer(const_cast<IDevice *>(device));
 }
 
+QString DeviceManager::hostKeysFilePath()
+{
+    return settingsFilePath(QLatin1String("/ssh-hostkeys")).toString();
+}
+
 } // namespace ProjectExplorer
 
 
@@ -463,11 +478,11 @@ void ProjectExplorerPlugin::testDeviceManager()
     QVERIFY(!mgr->find(dev->id()));
     const int oldDeviceCount = mgr->deviceCount();
 
-    QSignalSpy deviceAddedSpy(mgr, SIGNAL(deviceAdded(Core::Id)));
-    QSignalSpy deviceRemovedSpy(mgr, SIGNAL(deviceRemoved(Core::Id)));
-    QSignalSpy deviceUpdatedSpy(mgr, SIGNAL(deviceUpdated(Core::Id)));
-    QSignalSpy deviceListReplacedSpy(mgr, SIGNAL(deviceListReplaced()));
-    QSignalSpy updatedSpy(mgr, SIGNAL(updated()));
+    QSignalSpy deviceAddedSpy(mgr, &DeviceManager::deviceAdded);
+    QSignalSpy deviceRemovedSpy(mgr, &DeviceManager::deviceRemoved);
+    QSignalSpy deviceUpdatedSpy(mgr, &DeviceManager::deviceUpdated);
+    QSignalSpy deviceListReplacedSpy(mgr, &DeviceManager::deviceListReplaced);
+    QSignalSpy updatedSpy(mgr, &DeviceManager::updated);
 
     mgr->addDevice(dev);
     QCOMPARE(mgr->deviceCount(), oldDeviceCount + 1);
@@ -515,7 +530,7 @@ void ProjectExplorerPlugin::testDeviceManager()
     dev3->setDisplayName(dev->displayName());
     mgr->addDevice(dev3);
     QCOMPARE(mgr->deviceAt(mgr->deviceCount() - 1)->displayName(),
-             QString(dev3->displayName() + QLatin1String("2")));
+             QString(dev3->displayName() + QLatin1Char('2')));
     QCOMPARE(deviceAddedSpy.count(), 1);
     QCOMPARE(deviceRemovedSpy.count(), 0);
     QCOMPARE(deviceUpdatedSpy.count(), 0);
