@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -35,6 +30,7 @@
 #include "qbsproject.h"
 #include "qbsprojectmanagerconstants.h"
 #include "qbsprojectmanagerplugin.h"
+#include "qbsprojectmanagersettings.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/messagemanager.h>
@@ -46,6 +42,7 @@
 #include <qtsupport/baseqtversion.h>
 #include <qtsupport/qtkitinformation.h>
 
+#include <QCryptographicHash>
 #include <QVariantMap>
 
 #include <qbs.h>
@@ -66,7 +63,6 @@ QbsManager *QbsManager::m_instance = 0;
 QbsManager::QbsManager() :
     m_defaultPropertyProvider(new DefaultPropertyProvider)
 {
-    m_settings = new qbs::Settings(Core::ICore::userResourcePath());
     m_instance = this;
 
     setObjectName(QLatin1String("QbsProjectManager"));
@@ -78,6 +74,8 @@ QbsManager::QbsManager() :
             &QbsManager::handleKitUpdate);
     connect(ProjectExplorer::KitManager::instance(), &ProjectExplorer::KitManager::kitRemoved, this,
             &QbsManager::handleKitRemoval);
+    connect(&QbsProjectManagerSettings::instance(), &QbsProjectManagerSettings::settingsBaseChanged,
+            this, &QbsManager::updateAllProfiles);
 
     m_logSink = new QbsLogSink(this);
     int level = qbs::LoggerWarning;
@@ -125,12 +123,12 @@ QString QbsManager::profileForKit(const ProjectExplorer::Kit *k)
     if (!k)
         return QString();
     updateProfileIfNecessary(k);
-    return m_settings->value(qtcProfilePrefix() + k->id().toString()).toString();
+    return settings()->value(qtcProfilePrefix() + k->id().toString()).toString();
 }
 
 void QbsManager::setProfileForKit(const QString &name, const ProjectExplorer::Kit *k)
 {
-    m_settings->setValue(qtcProfilePrefix() + k->id().toString(), name);
+    settings()->setValue(qtcProfilePrefix() + k->id().toString(), name);
 }
 
 void QbsManager::updateProfileIfNecessary(const ProjectExplorer::Kit *kit)
@@ -141,6 +139,22 @@ void QbsManager::updateProfileIfNecessary(const ProjectExplorer::Kit *kit)
         addProfileFromKit(kit);
 }
 
+void QbsManager::updateAllProfiles()
+{
+    foreach (const auto * const kit, ProjectExplorer::KitManager::kits())
+        addProfileFromKit(kit);
+}
+
+qbs::Settings *QbsManager::settings()
+{
+    if (!m_settings
+            || m_settings->baseDirectory() != QbsProjectManagerSettings::qbsSettingsBaseDir()) {
+        delete m_settings;
+        m_settings = new qbs::Settings(QbsProjectManagerSettings::qbsSettingsBaseDir());
+    }
+    return m_settings;
+}
+
 void QbsManager::addProfile(const QString &name, const QVariantMap &data)
 {
     qbs::Profile profile(name, settings());
@@ -194,7 +208,8 @@ void QbsManager::addQtProfileFromKit(const QString &profileName, const ProjectEx
 void QbsManager::addProfileFromKit(const ProjectExplorer::Kit *k)
 {
     const QString name = QString::fromLatin1("qtc_%1_%2").arg(k->fileSystemFriendlyName().left(8),
-            QString::number(k->id().uniqueIdentifier(), 16));
+            QString::fromLatin1(QCryptographicHash::hash(k->id().name(),
+                                                         QCryptographicHash::Sha1).toHex().left(8)));
     qbs::Profile(name, settings()).removeProfile();
     setProfileForKit(name, k);
     addQtProfileFromKit(name, k);
@@ -220,9 +235,9 @@ void QbsManager::handleKitRemoval(ProjectExplorer::Kit *kit)
 {
     m_kitsToBeSetupForQbs.removeOne(kit);
     const QString key = qtcProfilePrefix() + kit->id().toString();
-    const QString profileName = m_settings->value(key).toString();
-    m_settings->remove(key);
-    qbs::Profile(profileName, m_settings).removeProfile();
+    const QString profileName = settings()->value(key).toString();
+    settings()->remove(key);
+    qbs::Profile(profileName, settings()).removeProfile();
 }
 
 } // namespace Internal
