@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,17 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -454,13 +454,12 @@ PropertyNameList getSignals(const ObjectValue *objectValue, const ContextPtr &co
 
     signalList.append(processor.signalList());
 
+    PrototypeIterator prototypeIterator(objectValue, context);
+    QList<const ObjectValue *> objects = prototypeIterator.all();
+
     if (!local) {
-        const ObjectValue* prototype = objectValue->prototype(context);
-
-        if (prototype == objectValue)
-            return signalList;
-
-        signalList.append(getSignals(prototype, context));
+        foreach (const ObjectValue *prototype, objects)
+            signalList.append(getSignals(prototype, context, true));
     }
 
     return signalList;
@@ -1520,7 +1519,7 @@ bool NodeMetaInfo::isSubclassOf(const TypeName &type, int majorVersion, int mino
 
 bool NodeMetaInfo::isGraphicalItem() const
 {
-    return isSubclassOf("QtQuick.Item", -1, -1) || isSubclassOf("QtQuick.Window.Window", -1, -1);
+    return isSubclassOf("QtQuick.Item") || isSubclassOf("QtQuick.Window.Window");
 }
 
 void NodeMetaInfo::clearCache()
@@ -1530,25 +1529,25 @@ void NodeMetaInfo::clearCache()
 
 bool NodeMetaInfo::isLayoutable() const
 {
-    if (isSubclassOf("<cpp>.QDeclarativeBasePositioner", -1, -1))
+    if (isSubclassOf("<cpp>.QDeclarativeBasePositioner"))
         return true; //QtQuick 1
 
-    return isSubclassOf("QtQuick.Positioner", -1, -1)
-            || isSubclassOf("QtQuick.Layouts.Layout", -1, -1)
-            || isSubclassOf("QtQuick.Controls.SplitView", -1, -1);
+    return isSubclassOf("QtQuick.Positioner")
+            || isSubclassOf("QtQuick.Layouts.Layout")
+            || isSubclassOf("QtQuick.Controls.SplitView");
 }
 
 bool NodeMetaInfo::isView() const
 {
     return isValid() &&
-            (isSubclassOf("QtQuick.ListView", -1, -1) ||
-             isSubclassOf("QtQuick.GridView", -1, -1) ||
-             isSubclassOf("QtQuick.PathView", -1, -1));
+            (isSubclassOf("QtQuick.ListView") ||
+             isSubclassOf("QtQuick.GridView") ||
+             isSubclassOf("QtQuick.PathView"));
 }
 
 bool NodeMetaInfo::isTabView() const
 {
-    return isSubclassOf("QtQuick.Controls.TabView", -1, -1);
+    return isSubclassOf("QtQuick.Controls.TabView");
 }
 
 } // namespace QmlDesigner
